==> tutti-ui-swing-1.4-sources.jar: fr/ifremer/tutti/ui/swing/content/config/SampleOrderEditorUIHandler.java <==
package fr.ifremer.tutti.ui.swing.content.config;

/*
 * #%L
 * Tutti :: UI
 * $Id: SampleOrderEditorUIHandler.java 694 2013-03-27 21:56:27Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-1.4/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/content/config/SampleOrderEditorUIHandler.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import fr.ifremer.tutti.persistence.entities.referential.Caracteristic;
import fr.ifremer.tutti.service.DecoratorService;
import fr.ifremer.tutti.ui.swing.TuttiUIContext;
import jaxx.runtime.SwingUtil;
import jaxx.runtime.swing.ComponentMover;
import jaxx.runtime.swing.ComponentResizer;
import jaxx.runtime.swing.model.JaxxDefaultListModel;
import jaxx.runtime.swing.renderer.MultiDecoratorListCellRenderer;
import org.nuiton.util.decorator.JXPathDecorator;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JRootPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.List;

import static org.nuiton.i18n.I18n._;

/**
 * @author tchemit <chemit@codelutin.com>
 * @since 1.1
 */
public class SampleOrderEditorUIHandler {

    private static final int DEFAULT_EDITOR_WIDTH = 300;

    private static final int DEFAULT_EDITOR_HEIGHT = 200;

    public static final String CLOSE_DIALOG_ACTION = "closeDialog";

    public static final String SHOW_DIALOG_ACTION = "showDialog";

    protected final TuttiUIContext context;

    private final SampleOrderEditorUI ui;

    public SampleOrderEditorUIHandler(TuttiUIContext context,
                                      SampleOrderEditorUI ui) {
        this.context = context;
        this.ui = ui;
    }

    protected void beforeInitUI() {
    }

    protected void afterInitUI() {

        ui.setResizable(true);

        ui.setSize(DEFAULT_EDITOR_WIDTH, DEFAULT_EDITOR_HEIGHT);
        ui.pack();

        ComponentResizer cr = new ComponentResizer();
        cr.registerComponent(ui);
        ComponentMover cm = new ComponentMover();
        cm.setDragInsets(cr.getDragInsets());
        cm.registerComponent(ui);

        JRootPane rootPane = ui.getRootPane();

        KeyStroke shortcutClosePopup =
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);

        rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
                shortcutClosePopup, CLOSE_DIALOG_ACTION);

        closeAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;

            @Override
            public void actionPerformed(ActionEvent e) {
                ui.dispose();
                ui.setVisible(false);
            }
        };

        openAction = new AbstractAction() {

            private static final long serialVersionUID = 1L;

            @Override
            public void actionPerformed(ActionEvent e) {
                ui.setVisible(true);
            }
        };

        ImageIcon actionIcon = SwingUtil.createActionIcon("close-dialog");
        closeAction.putValue(Action.SMALL_ICON, actionIcon);
        closeAction.putValue(Action.LARGE_ICON_KEY, actionIcon);
        closeAction.putValue(Action.ACTION_COMMAND_KEY, "close");
        closeAction.putValue(Action.NAME, "close");
        closeAction.putValue(Action.SHORT_DESCRIPTION, _("tutti.commentEditor.action.close.tip"));

        rootPane.getActionMap().put(CLOSE_DIALOG_ACTION, closeAction);
        rootPane.getActionMap().put(SHOW_DIALOG_ACTION, openAction);

        JButton closeButton = new JButton(closeAction);
        closeButton.setText(null);
        closeButton.setFocusPainted(false);
        closeButton.setRequestFocusEnabled(false);
        closeButton.setFocusable(false);

        JToolBar jToolBar = new JToolBar();
        jToolBar.setOpaque(false);
        jToolBar.add(closeAction);
        jToolBar.setBorderPainted(false);
        jToolBar.setFloatable(false);
        ui.getSampleOrderEditorTopPanel().setRightDecoration(jToolBar);

        JXPathDecorator<Caracteristic> decorator =
                (JXPathDecorator<Caracteristic>) context.getDecoratorService().getDecoratorByType(
                        Caracteristic.class, DecoratorService.CARACTERISTIC_PARAMETER_ONLY);
        ui.getList().setCellRenderer(new MultiDecoratorListCellRenderer(decorator));

        ui.getList().addKeyListener(new KeyAdapter() {

            JaxxDefaultListModel<Caracteristic> listModel = ui.getListModel();

            @Override
            public void keyReleased(KeyEvent e) {
                super.keyReleased(e);
                JList list = (JList) e.getSource();

                if (!list.isSelectionEmpty()) {

                    int selectedIndex = list.getSelectedIndex();

                    Integer newSelectedIndex = null;

                    if (e.getKeyCode() == KeyEvent.VK_DOWN) {

                        if (selectedIndex < listModel.size()) {

                            // can move to down
                            newSelectedIndex = selectedIndex - 1;

                            Caracteristic remove = listModel.remove(newSelectedIndex);

                            listModel.insertElementAt(remove, selectedIndex);
                        }
                    } else if (e.getKeyCode() == KeyEvent.VK_UP) {

                        if (selectedIndex >= 0) {

                            // can move to up
                            newSelectedIndex = selectedIndex + 1;

                            Caracteristic remove = listModel.remove(newSelectedIndex);
                            listModel.insertElementAt(remove, selectedIndex);
                        }
                    }

                    if (newSelectedIndex != null) {
                        list.setSelectedIndex(selectedIndex);
                    }
                }

            }
        });
    }

    protected Action closeAction;

    protected Action openAction;

    public void closeEditor() {

        closeAction.actionPerformed(null);
    }

    public void openEditor(JComponent component) {

        if (component != null) {
            place(component);
        }
        openAction.actionPerformed(null);
    }

    public void init() {

        List<Caracteristic> bean = ui.getBean();
        ui.getListModel().setAllElements(bean);
    }

    public void place(JComponent component) {
        // Computes the location of bottom left corner of the cell
        Component comp = component;
        int x = 0;
        int y = component.getHeight();
        while (comp != null) {
            x += comp.getX();
            y += comp.getY();
            comp = comp.getParent();
        }

//        ui.pack();
        // if the editor is too big on the right,
        // then align its right side to the right side of the cell
        if (x + ui.getWidth() > ui.getOwner().getX() + ui.getOwner().getWidth()) {
            x = x - ui.getWidth() + component.getWidth();
        }
        ui.setLocation(x, y);
    }
}

==> tutti-ui-swing-1.4.1-sources.jar: fr/ifremer/tutti/ui/swing/content/config/SampleOrderEditorUIHandler.java <==
package fr.ifremer.tutti.ui.swing.content.config;

/*
 * #%L
 * Tutti :: UI
 * $Id: SampleOrderEditorUIHandler.java 694 2013-03-27 21:56:27Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-1.4.1/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/content/config/SampleOrderEditorUIHandler.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import fr.ifremer.tutti.persistence.entities.referential.Caracteristic;
import fr.ifremer.tutti.service.DecoratorService;
import fr.ifremer.tutti.ui.swing.TuttiUIContext;
import jaxx.runtime.SwingUtil;
import jaxx.runtime.swing.ComponentMover;
import jaxx.runtime.swing.ComponentResizer;
import jaxx.runtime.swing.model.JaxxDefaultListModel;
import jaxx.runtime.swing.renderer.MultiDecoratorListCellRenderer;
import org.nuiton.util.decorator.JXPathDecorator;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JRootPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.List;

import static org.nuiton.i18n.I18n._;

/**
 * @author tchemit <chemit@codelutin.com>
 * @since 1.1
 */
public class SampleOrderEditorUIHandler {

    private static final int DEFAULT_EDITOR_WIDTH = 300;

    private static final int DEFAULT_EDITOR_HEIGHT = 200;

    public static final String CLOSE_DIALOG_ACTION = "closeDialog";

    public static final String SHOW_DIALOG_ACTION = "showDialog";

    protected final TuttiUIContext context;

    private final SampleOrderEditorUI ui;

    public SampleOrderEditorUIHandler(TuttiUIContext context,
                                      SampleOrderEditorUI ui) {
        this.context = context;
        this.ui = ui;
    }

    protected void beforeInitUI() {
    }

    protected void afterInitUI() {

        ui.setResizable(true);

        ui.setSize(DEFAULT_EDITOR_WIDTH, DEFAULT_EDITOR_HEIGHT);
        ui.pack();

        ComponentResizer cr = new ComponentResizer();
        cr.registerComponent(ui);
        ComponentMover cm = new ComponentMover();
        cm.setDragInsets(cr.getDragInsets());
        cm.registerComponent(ui);

        JRootPane rootPane = ui.getRootPane();

        KeyStroke shortcutClosePopup =
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);

        rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
                shortcutClosePopup, CLOSE_DIALOG_ACTION);

        closeAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;

            @Override
            public void actionPerformed(ActionEvent e) {
                ui.dispose();
                ui.setVisible(false);
            }
        };

        openAction = new AbstractAction() {

            private static final long serialVersionUID = 1L;

            @Override
            public void actionPerformed(ActionEvent e) {
                ui.setVisible(true);
            }
        };

        ImageIcon actionIcon = SwingUtil.createActionIcon("close-dialog");
        closeAction.putValue(Action.SMALL_ICON, actionIcon);
        closeAction.putValue(Action.LARGE_ICON_KEY, actionIcon);
        closeAction.putValue(Action.ACTION_COMMAND_KEY, "close");
        closeAction.putValue(Action.NAME, "close");
        closeAction.putValue(Action.SHORT_DESCRIPTION, _("tutti.commentEditor.action.close.tip"));

        rootPane.getActionMap().put(CLOSE_DIALOG_ACTION, closeAction);
        rootPane.getActionMap().put(SHOW_DIALOG_ACTION, openAction);

        JButton closeButton = new JButton(closeAction);
        closeButton.setText(null);
        closeButton.setFocusPainted(false);
        closeButton.setRequestFocusEnabled(false);
        closeButton.setFocusable(false);

        JToolBar jToolBar = new JToolBar();
        jToolBar.setOpaque(false);
        jToolBar.add(closeAction);
        jToolBar.setBorderPainted(false);
        jToolBar.setFloatable(false);
        ui.getSampleOrderEditorTopPanel().setRightDecoration(jToolBar);

        JXPathDecorator<Caracteristic> decorator =
                (JXPathDecorator<Caracteristic>) context.getDecoratorService().getDecoratorByType(
                        Caracteristic.class, DecoratorService.CARACTERISTIC_PARAMETER_ONLY);
        ui.getList().setCellRenderer(new MultiDecoratorListCellRenderer(decorator));

        ui.getList().addKeyListener(new KeyAdapter() {

            JaxxDefaultListModel<Caracteristic> listModel = ui.getListModel();

            @Override
            public void keyReleased(KeyEvent e) {
                super.keyReleased(e);
                JList list = (JList) e.getSource();

                if (!list.isSelectionEmpty()) {

                    int selectedIndex = list.getSelectedIndex();

                    Integer newSelectedIndex = null;

                    if (e.getKeyCode() == KeyEvent.VK_DOWN) {

                        if (selectedIndex < listModel.size()) {

                            // can move to down
                            newSelectedIndex = selectedIndex - 1;

                            Caracteristic remove = listModel.remove(newSelectedIndex);

                            listModel.insertElementAt(remove, selectedIndex);
                        }
                    } else if (e.getKeyCode() == KeyEvent.VK_UP) {

                        if (selectedIndex >= 0) {

                            // can move to up
                            newSelectedIndex = selectedIndex + 1;

                            Caracteristic remove = listModel.remove(newSelectedIndex);
                            listModel.insertElementAt(remove, selectedIndex);
                        }
                    }

                    if (newSelectedIndex != null) {
                        list.setSelectedIndex(selectedIndex);
                    }
                }

            }
        });
    }

    protected Action closeAction;

    protected Action openAction;

    public void closeEditor() {

        closeAction.actionPerformed(null);
    }

    public void openEditor(JComponent component) {

        if (component != null) {
            place(component);
        }
        openAction.actionPerformed(null);
    }

    public void init() {

        List<Caracteristic> bean = ui.getBean();
        ui.getListModel().setAllElements(bean);
    }

    public void place(JComponent component) {
        // Computes the location of bottom left corner of the cell
        Component comp = component;
        int x = 0;
        int y = component.getHeight();
        while (comp != null) {
            x += comp.getX();
            y += comp.getY();
            comp = comp.getParent();
        }

//        ui.pack();
        // if the editor is too big on the right,
        // then align its right side to the right side of the cell
        if (x + ui.getWidth() > ui.getOwner().getX() + ui.getOwner().getWidth()) {
            x = x - ui.getWidth() + component.getWidth();
        }
        ui.setLocation(x, y);
    }
}
